#include "mytypes.h"

typedef struct {

  int CTRL_Y        :1;  /* Bit 0 - User can not use CTRL-Y                 */
  int default_com   :1;  /* Bit 1 - User is restricted to default command   */
  int set_pass      :1;  /* Bit 2 - SET PASSWORD command is disabled        */
  int prevent_login :1;  /* Bit 3 - Prevent user from changing any login    */
  int user_account  :1;  /* Bit 4 - User account is disabled                */
  int welcome       :1;  /* Bit 5 - User will not receive the login message */
  int new_mail      :1;  /* Bit 6 - Announcement of new mail is suppressed  */
  int mail_delivery :1;  /* Bit 7 - Mail delivery to user is disabled       */

} L_Flags_0;

typedef struct {

  int gen_passed    :1;   /* Bit 0 - User is required to use generated passwd */
  int primary_pass  :1;   /* Bit 1 - Primary password is expired              */
  int secondary_pass:1;   /* Bit 2 - Secondary password is expired            */
  int act_audited   :1;   /* Bit 3 - All actions are audited                  */
  int last_login    :1;   /* Bit 4 - User will not receive last login messages*/
  int existing_proc :1;   /* Bit 5 - User can not reconnect to existing proces*/
  int user_login    :1;   /* Bit 6 - User can only login to terminals defined */
  int change_pass   :1;   /* Bit 7 - User is required to change expired passwd*/

} L_Flags_1;

typedef struct {

  int captive_acc   :1;   /* Bit 0 - User is restricted to captive account  */
  int user_execut   :1;   /* Bit 1 - Prevent user from executing RUN at DCL */
  int future_use    :6;   /* Bits 2-7 Reserved for future use               */

} L_Flags_2;

typedef struct {

  dword Record_Header;              /* Record Header                        */
  char  Username[32];               /* Username (loginid)                   */
  word  Member_UIC;                 /* Mem UIC decimal   1 = 0100           */
                                    /* Mem UIC decimal  10 = 0A00           */
                                    /* Mem UIC decimal 256 = FF01           */
  word  Group_UIC;                  /* Group UIC                            */
  byte  Nulls[12];                  /* Nulls                                */
  char  Account_name[32];           /* Account name                         */
  byte  length_owner;               /* length of owner                      */
  char  Owner[31];                  /* Owner                                */
  byte  length_device;              /* length of device                     */
  char  Device[31];                 /* Device (default disk device)         */
  byte  length_default;             /* length of default (SYS$LOGIN)        */
  char  Default[63];                /* Default (SYS$LOGIN) directory        */
  byte  length_command;             /* length of default login command      */
  char  Default_command[63];        /* Default login command file           */
  byte  length_CLI;                 /* length of default CLI                */
  char  Default_CLI[31];            /* Default CLI                          */
  byte  length_CLI_tables;          /* length of user defined CLI tables    */
  char  User_CLI_table[31];         /* User defined CLI table name          */
  byte  Passwd1[8];                 /* Encrypted primary password           */
  byte  Passwd2[8];                 /* Encrypted secondary password         */
  word  Number_login_fails;         /* Number of login fails                */
  word  Passwd_salt;                /* Password encryption salt             */
  byte  Encr_algo1;                 /* Encryption algorithm code - pass1    */
  byte  Encr_algo2;                 /* Encryption algorithm code - pass2    */
  byte  Pass_minimum_length;        /* Password minimum length              */
  byte  Filler1;                    /* Filler                               */
  byte  Account_ex_date[8];         /* Account expiration date              */
  byte  Password_lifetime[8];       /* Password lifetime                    */
  byte  Pass1_change_date[8];       /* Primary password change date/time    */
  byte  Pass2_change_date[8];       /* Secondary password change date/time  */
  byte  Last_interactive[8];        /* Last interactive login date/time     */
  byte  Last_non_interactive[8];    /* Last non-interactive login date/time */
  byte  Authorize_priviledges[8];   /* Authorize priviledges                */
  byte  Default_priviledges[8];     /* Default priviledges                  */
  byte  Filler2[40];                /* Filler                               */
  byte  Login_Flags_0;              /* Login Flags bits byte 0              */
  byte  Login_Flags_1;              /* Login Flags bits byte 1              */
  byte  Login_Flags_2;              /* Login Flags bits byte 2              */
  byte  Login_Flags_3;              /* Login Flags bits byte 3              */
  byte  Network_primary_days[3];    /* Network access bytes - primary days  */
  byte  Network_seconday_days[3];   /* Network access bytes - seconday days */
  byte  Batch_primary_days[3];      /* Batch access bytes - primary days    */
  byte  Batch_seconday_days[3];     /* Batch access bytes - seconday days   */
  byte  Local_primary_days[3];      /* Local access bytes - primary days    */
  byte  Local_seconday_days[3];     /* Local access bytes - seconday days   */
  byte  Dialup_primary_days[3];     /* Dialup access bytes - primary days   */
  byte  Dialup_secondary_days[3];   /* Dialup access bytes - secondary days */
  byte  Remote_primary_days[3];     /* Remote access bytes - primary days   */
  byte  Remote_seconday_days[3];    /* Remote access bytes - seconday days  */
  byte  Filler3[12];                /* Filler                               */
  byte  Prime_days;                 /* Prime days                           */
  byte  Filler4;                    /* Filler                               */
  byte  Default_base_priority;      /* Default base priority                */
  byte  Maximum_job_queue_priority; /* Maximum job queue priority           */
  word  Active_process_limit;       /* Active process limit                 */
  word  Max_number_jobs;            /* Max. number of interactive jobs      */
  word  Detached_process_limit;     /* Detached process limit               */
  word  Subprocess_creation_limit;  /* Subprocess creation limit            */
  word  Buffered_IO_count;          /* Buffered I/O count                   */
  word  Timer_queue_entry_limit;    /* Timer queue entry limit              */
  word  AST_queue_limit;            /* AST queue limit                      */
  word  Lock_queue_limit;           /* Lock queue limit                     */
  word  Open_file_limit;            /* Open file limit                      */
  word  unknown1;
  word  Shared_file_limit;          /* Shared file limit                    */
  dword Working_set_quota;          /* Working set quota                    */
  dword unknown2;
  dword Working_set_extent;         /* Working set extent                   */
  dword Paging_file_quota;          /* Paging file quota                    */
  dword Maximum_CPU_time_limit;     /* Maximum CPU time limit (in 10-msec)  */
  dword Buffered_IO_byte_limit;     /* Buffered I/O byte limit              */
  dword Paged_buffer_IO_count_limit;/* Paged buffer I/O byte count limit    */
  dword Initial_byte_quota;         /* Initial byte quota                   */
  byte  Filler5[72];                /* Filler                               */

} UAF;
